@@ -26,14 +26,22 @@
 #endif
 
 #include <stdint.h>
-
 #include <ctype.h>
-#include "track.h"
+#include <math.h>
+#include <string.h>
+#include <messages.h>
+
 #include "ccurve.h"
-#include "cstraigh.h"
 #include "compound.h"
+#include "cstraigh.h"
+#include "custom.h"
+#include "fileio.h"
 #include "i18n.h"
 
+#include "param.h"
+#include "track.h"
+#include "utility.h"
+
 #define TURNOUTDESIGNER			"CTURNOUT DESIGNER"
 
 
@@ -1414,7 +1422,7 @@ static void NewTurnPrint(
 			wDrawString( newTurnout_d.d, POSX(3.0),
 						POSY(6.25), 0.0, message, fp, 30,
 						wDrawColorBlack, 0 );
-			sprintf( message, "%s %d x %d (of %d x %d)", _("Page"), i+1, j+1, ii, jj );
+			sprintf( message, _("%s %d x %d (of %d x %d)"), _("Page"), i+1, j+1, ii, jj );
 			wDrawString( newTurnout_d.d, POSX(3.0),
 						POSY(5.75), 0.0, message, fp, 20,
 						wDrawColorBlack, 0 );
@@ -2176,14 +2184,14 @@ EXPORT BOOL_T WriteSegs(
 		case SEG_CRVLIN:
 			rc &= fprintf( f, "\t%c %ld %0.6f %0.6f %0.6f %0.6f %0.6f %0.6f\n",
 				segs[i].type, (segs[i].type==SEG_CRVTRK?0:roadbedColorRGB), segs[i].width,
-				segs[i].u.c.radius,
+				fabs(segs[i].u.c.radius),
 				segs[i].u.c.center.x, segs[i].u.c.center.y,
 				segs[i].u.c.a0, segs[i].u.c.a1 )>0;
 			break;
 		case SEG_FILCRCL:
 			rc &= fprintf( f, "\t%c %ld %0.6f %0.6f %0.6f %0.6f\n",
 				segs[i].type, roadbedColorRGB, segs[i].width,
-				segs[i].u.c.radius,
+				fabs(segs[i].u.c.radius),
 				segs[i].u.c.center.x, segs[i].u.c.center.y )>0;
 			break;
 		case SEG_POLY:
