@@ -21,6 +21,7 @@
  */
 
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdio.h>
 #ifndef WINDOWS
 #include <unistd.h>
@@ -45,12 +46,17 @@
 #include <stdarg.h>
 #include <locale.h>
 #include <wlib.h>
-#include "track.h"
+
 #include "common.h"
-#include "utility.h"
-#include "misc.h"
 #include "compound.h"
+#include "custom.h"
+#include "fileio.h"
 #include "i18n.h"
+#include "messages.h"
+#include "misc.h"
+#include "param.h"
+#include "track.h"
+#include "utility.h"
 
 
 /* Bogus reg vars */
@@ -65,102 +71,8 @@ EXPORT char *PREFSECT = "DialogItem";
 static int paramCheckErrorCount = 0;
 static BOOL_T paramCheckShowErrors = FALSE;
 
-static int log_hotspot;
-static int hotspotOffsetX = 5;
-static int hotspotOffsetY = 19;
-
 static int log_paramLayout;
 
-
-#ifdef LATER
-/*****************************************************************************
- *
- *	Colors
- *
- */
-
-typedef struct {
-		long rgb;
-		char * name;
-		wDrawColor color;
-		} colorTab_t[];
-
-static colorTab_t colorTab = {
-				{ wRGB(	 0,	 0,	 0), N_("Black") },
-
-				{ wRGB(	 0,	 0,128), N_("Dark Blue") },
-				{ wRGB( 70,130,180), N_("Steel Blue") },
-				{ wRGB( 65,105,225), N_("Royal Blue") },
-				{ wRGB(	 0,	 0,255), N_("Blue") },
-				{ wRGB(	 0,191,255), N_("Deep Sky Blue") },
-				{ wRGB(125,206,250), N_("Light Sky Blue") },
-				{ wRGB(176,224,230), N_("Powder Blue") },
-
-				{ wRGB(	 0,128,128), N_("Dark Aqua") },
-				{ wRGB(127,255,212), N_("Aquamarine") },
-				{ wRGB(	 0,255,255), N_("Aqua") },
-
-				{ wRGB(	 0,128,	 0), N_("Dark Green") },
-				{ wRGB( 34,139, 34), N_("Forest Green") },
-				{ wRGB( 50,205, 50), N_("Lime Green") },
-				{ wRGB(	 0,255,	 0), N_("Green") },
-				{ wRGB(124,252,	 0), N_("Lawn Green") },
-				{ wRGB(152,251,152), N_("Pale Green") },
-
-				{ wRGB(128,128,	 0), N_("Dark Yellow") },
-				{ wRGB(255,127, 80), N_("Coral") },
-				{ wRGB(255,165,	 0), N_("Orange") },
-				{ wRGB(255,215,	 0), N_("Gold") },
-				{ wRGB(255,255,	 0), N_("Yellow") },
-
-				{ wRGB(139, 69, 19), N_("Saddle Brown") },
-				{ wRGB(165, 42, 42), N_("Brown") },
-				{ wRGB(210,105, 30), N_("Chocolate") },
-				{ wRGB(188,143,143), N_("Rosy Brown") },
-				{ wRGB(210,180,140), N_("Tan") },
-				{ wRGB(245,245,220), N_("Beige") },
-
-
-				{ wRGB(128,	 0,	 0), N_("Dark Red") },
-				{ wRGB(255, 99, 71), N_("Tomato") },
-				{ wRGB(255,	 0,	 0), N_("Red") },
-				{ wRGB(255,105,180), N_("Hot Pink") },
-				{ wRGB(255,192,203), N_("Pink") },
-
-				{ wRGB(128,	 0,128), N_("Dark Purple") },
-				{ wRGB(176, 48, 96), N_("Maroon") },
-				{ wRGB(160, 32,240), N_("Purple2") },
-				{ wRGB(255,	 0,255), N_("Purple") },
-				{ wRGB(238,130,238), N_("Violet") },
-
-				{ wRGB( 64, 64, 64), N_("Dark Gray") },
-				{ wRGB(128,128,128), N_("Gray") },
-				{ wRGB(192,192,192), N_("Light Gray") } };
-static wIcon_p colorTabBitMaps[ sizeof colorTab/sizeof colorTab[0] ];
-#include "bitmaps/square10.xbm"
-
-static BOOL_T colorTabInitted = FALSE;
-
-static void InitColorTab( void )
-{
-	wIndex_t inx;
-	for ( inx=0; inx<COUNT(colorTab); inx++ )
-		colorTab[inx].color = wDrawFindColor( colorTab[inx].rgb );
-	colorTabInitted = TRUE;
-}
-
-
-static wIndex_t ColorTabLookup( wDrawColor color )
-{
-	wIndex_t inx;
-	if (!colorTabInitted)
-		InitColorTab();
-	for (inx = 0; inx < sizeof colorTab/sizeof colorTab[0]; inx++ )
-		if (colorTab[inx].color == color)
-			return inx;
-	return 0;
-}
-#endif
 
 /*****************************************************************************
  *
@@ -258,88 +170,119 @@ static int GetNumberStr( char ** cpp, FLOAT_T * numP, BOOL_T * hasFract )
 }
 //extern wIndex_t distanceFormatInx;   // distanceFormatInx
 
-static BOOL_T GetDistance( char ** cpp, FLOAT_T * distP )
+static BOOL_T GetDistance(char ** cpp, FLOAT_T * distP)
 {
-	FLOAT_T n1, n2;
-	BOOL_T neg = FALSE;
-	BOOL_T hasFract;
-	BOOL_T expectInch = FALSE;
-	long distanceFormat;
+    FLOAT_T n1, n2;
+    BOOL_T neg = FALSE;
+    BOOL_T hasFract;
+    BOOL_T expectInch = FALSE;
+    long distanceFormat;
 
-	while ( isspace((unsigned char)**cpp) ) (*cpp)++;
-	if ( (*cpp)[0] == '\0' ) {
-		*distP = 0.0;
-		return TRUE;
-	}
-	if ( (*cpp)[0] == '-' ) {
-		neg = TRUE;
-		(*cpp)++;
-	}
-	if ( !GetNumberStr( cpp, &n1, &hasFract ) ) return FALSE;
+    while (isspace((unsigned char)**cpp)) {
+        (*cpp)++;
+    }
 
-	distanceFormat = GetDistanceFormat();
+    if ((*cpp)[0] == '\0') {
+        *distP = 0.0;
+        return TRUE;
+    }
 
+    if ((*cpp)[0] == '-') {
+        neg = TRUE;
+        (*cpp)++;
+    }
 
-	if ( (*cpp)[0] == '\0' ) { /* EOL */
-		if ( units==UNITS_METRIC )
-		{
-		   n1 = n1/2.54;
-		   if ((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_MM)
-			   n1 /= 10;
-		   if ((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_M)
-			   n1 *= 100;
-		} else {
-			if (((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_SHRT) || ((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_LONG))
-				n1 *= 12;
-		}
-		if ( neg )
-			n1 = -n1;
-		*distP = n1;
-		return TRUE;
-	}
-	if ( (*cpp)[0] == '\'' ) {
-		n1 *= 12.0;
-		(*cpp) += 1;
-		expectInch = !hasFract;
-	} else if ( tolower((unsigned char)(*cpp)[0]) == 'f' && tolower((unsigned char)(*cpp)[1]) == 't' ) {
-		n1 *= 12.0;
-		(*cpp) += 2;
-		expectInch = !hasFract;
-	} else if ( tolower((unsigned char)(*cpp)[0]) == 'c' && tolower((unsigned char)(*cpp)[1]) == 'm' ) {
-		n1 /= 2.54;
-		(*cpp) += 2;
-	} else if ( tolower((unsigned char)(*cpp)[0]) == 'm' && tolower((unsigned char)(*cpp)[1]) == 'm' ) {
-		n1 /= 25.4;
-		(*cpp) += 2;
-	} else if ( tolower((unsigned char)(*cpp)[0]) == 'm' ) {
-		n1 *= 100.0/2.54;
-		(*cpp) += 1;
-	} else if ( (*cpp)[0] == '"' ) {
-		(*cpp) += 1;
-	} else if ( tolower((unsigned char)(*cpp)[0]) == 'i' && tolower((unsigned char)(*cpp)[1]) == 'n' ) {
-		(*cpp) += 2;
-	} else {
-		getNumberError = N_("Invalid Units Indicator");
-		return FALSE;
-	}
-	while ( isspace((unsigned char)**cpp) ) (*cpp)++;
-	if ( expectInch && isdigit( (unsigned char)**cpp ) ) {
-		if ( !GetNumberStr( cpp, &n2, &hasFract ) ) return FALSE;
-		n1 += n2;
-		if ( (*cpp)[0] == '"' )
-			(*cpp) += 1;
-		else if ( tolower((unsigned char)(*cpp)[0]) == 'i' && tolower((unsigned char)(*cpp)[1]) == 'n' )
-			(*cpp) += 2;
-		while ( isspace((unsigned char)**cpp) ) (*cpp)++;
-	}
-	if ( **cpp ) {
-		getNumberError = N_("Expected End Of String");
-		return FALSE;
-	}
-	if ( neg )
-		n1 = -n1;
-	*distP = n1;
-	return TRUE;
+    if (!GetNumberStr(cpp, &n1, &hasFract)) {
+        return FALSE;
+    }
+
+    distanceFormat = GetDistanceFormat();
+
+    if ((*cpp)[0] == '\0') {   /* EOL */
+        if (units==UNITS_METRIC) {
+            n1 = n1/2.54;
+
+            if ((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_MM) {
+                n1 /= 10;
+            }
+
+            if ((distanceFormat & DISTFMT_FMT) == DISTFMT_FMT_M) {
+                n1 *= 100;
+            }
+        }
+
+        if (neg) {
+            n1 = -n1;
+        }
+
+        *distP = n1;
+        return TRUE;
+    }
+
+    if ((*cpp)[0] == '\'') {
+        n1 *= 12.0;
+        (*cpp) += 1;
+        expectInch = !hasFract;
+    } else if (tolower((unsigned char)(*cpp)[0]) == 'f' &&
+               tolower((unsigned char)(*cpp)[1]) == 't') {
+        n1 *= 12.0;
+        (*cpp) += 2;
+        expectInch = !hasFract;
+    } else if (tolower((unsigned char)(*cpp)[0]) == 'c' &&
+               tolower((unsigned char)(*cpp)[1]) == 'm') {
+        n1 /= 2.54;
+        (*cpp) += 2;
+    } else if (tolower((unsigned char)(*cpp)[0]) == 'm' &&
+               tolower((unsigned char)(*cpp)[1]) == 'm') {
+        n1 /= 25.4;
+        (*cpp) += 2;
+    } else if (tolower((unsigned char)(*cpp)[0]) == 'm') {
+        n1 *= 100.0/2.54;
+        (*cpp) += 1;
+    } else if ((*cpp)[0] == '"') {
+        (*cpp) += 1;
+    } else if (tolower((unsigned char)(*cpp)[0]) == 'i' &&
+               tolower((unsigned char)(*cpp)[1]) == 'n') {
+        (*cpp) += 2;
+    } else {
+        getNumberError = N_("Invalid Units Indicator");
+        return FALSE;
+    }
+
+    while (isspace((unsigned char)**cpp)) {
+        (*cpp)++;
+    }
+
+    if (expectInch && isdigit((unsigned char)**cpp)) {
+        if (!GetNumberStr(cpp, &n2, &hasFract)) {
+            return FALSE;
+        }
+
+        n1 += n2;
+
+        if ((*cpp)[0] == '"') {
+            (*cpp) += 1;
+        } else if (tolower((unsigned char)(*cpp)[0]) == 'i' &&
+                   tolower((unsigned char)(*cpp)[1]) == 'n') {
+            (*cpp) += 2;
+        }
+
+        while (isspace((unsigned char)**cpp)) {
+            (*cpp)++;
+        }
+    }
+
+    if (**cpp) {
+        getNumberError = N_("Expected End Of String");
+        return FALSE;
+    }
+
+    if (neg) {
+        n1 = -n1;
+    }
+
+    *distP = n1;
+    return TRUE;
 }
 
 
@@ -369,46 +312,57 @@ EXPORT FLOAT_T DecodeFloat(
 }
 
 
-EXPORT FLOAT_T DecodeDistance(
-		wString_p strCtrl,
-		BOOL_T * validP )
+FLOAT_T DecodeDistance(
+    wString_p strCtrl,
+    BOOL_T * validP)
 {
-	FLOAT_T valF;
-	char *cp0, *cp1, *cpN, c1;
+    FLOAT_T valF;
+    char *cp0, *cp1, *cpN, c1;
+    cp0 = cp1 = cpN = CAST_AWAY_CONST wStringGetValue(strCtrl);
+    cpN += strlen(cpN)-1;
 
-	cp0 = cp1 = cpN = CAST_AWAY_CONST wStringGetValue( strCtrl );
-	cpN += strlen(cpN)-1;
-	while (cpN > cp1 && isspace((unsigned char)*cpN) ) cpN--;
-	c1 = *cpN;
-	switch ( c1 ) {
-	case '=':
-	case 's':
-	case 'S':
-	case 'p':
-	case 'P':
-		*cpN = '\0';
-		break;
-	default:
-		cpN = NULL;
-	}
-	*validP = ( GetDistance( &cp1, &valF ) );
-	if ( cpN )
-		*cpN = c1;
-	if ( *validP ) {
-/*fprintf( stderr, "gd=%0.6f\n", valF );*/
-		if ( c1 == 's' || c1 == 'S' )
-			valF *= curScaleRatio;
-		else if ( c1 == 'p' || c1 == 'P' )
-			valF /= curScaleRatio;
-		if ( cpN )
-			wStringSetValue( strCtrl, FormatDistance( valF ) );
-	} else {
-/*fprintf( stderr, "Gd( @%s ) error=%s\n", cp1, getNumberError );*/
-		sprintf( decodeErrorStr, "%s @ %s", _(getNumberError), *cp1?cp1:_("End Of String") );
-		/*wStringSetHilight( strCtrl, cp1-cp0, -1 ); */
-		valF =	0.0;
-	}
-	return valF;
+    while (cpN > cp1 && isspace((unsigned char)*cpN)) {
+        cpN--;
+    }
+
+    c1 = *cpN;
+
+    switch (c1) {
+    case '=':
+    case 's':
+    case 'S':
+    case 'p':
+    case 'P':
+        *cpN = '\0';
+        break;
+
+    default:
+        cpN = NULL;
+    }
+
+    *validP = (GetDistance(&cp1, &valF));
+
+    if (cpN) {
+        *cpN = c1;
+    }
+
+    if (*validP) {
+        if (c1 == 's' || c1 == 'S') {
+            valF *= curScaleRatio;
+        } else if (c1 == 'p' || c1 == 'P') {
+            valF /= curScaleRatio;
+        }
+
+        if (cpN) {
+            wStringSetValue(strCtrl, FormatDistance(valF));
+        }
+    } else {
+        sprintf(decodeErrorStr, "%s @ %s", _(getNumberError),
+                *cp1?cp1:_("End Of String"));
+        valF =	0.0;
+    }
+
+    return valF;
 }
 
 
@@ -613,10 +567,6 @@ EXPORT void ParamLoadControl(
 			p->oldD.l = *(wIndex_t*)p->valueP;
 			break;
 		case PD_COLORLIST:
-#ifdef LATER
-			inx = ColorTabLookup( *(wDrawColor*)p->valueP );
-			wListSetIndex( (wList_p)p->control, inx );
-#endif
 			wColorSelectButtonSetColor( (wButton_p)p->control, *(wDrawColor*)p->valueP );
 			p->oldD.dc = *(wDrawColor*)p->valueP;
 			break;
@@ -636,10 +586,18 @@ EXPORT void ParamLoadControl(
 			p->oldD.f = tmpR;
 			break;
 		case PD_STRING:
-			wStringSetValue( (wString_p)p->control, (char*)p->valueP );
 			if (p->oldD.s)
 				MyFree( p->oldD.s );
-			p->oldD.s = MyStrdup( (char*)p->valueP );
+			if (p->context) {
+				p->oldD.s = MyMalloc((intptr_t)p->context);
+				strncpy(p->oldD.s, (char*)p->valueP, (intptr_t)p->context);
+				*(p->oldD.s + (intptr_t)p->context - 1) = '\0';
+				wStringSetValue((wString_p)p->control, (char*)p->oldD.s);
+			}
+			else {
+				p->oldD.s = MyStrdup((char *)p->valueP);
+				wStringSetValue((wString_p)p->control, (char*)p->valueP);
+			}
 			break;
 		case PD_MESSAGE:
 			wMessageSetValue( (wMessage_p)p->control, _((char*)p->valueP) );
@@ -652,6 +610,7 @@ EXPORT void ParamLoadControl(
 		case PD_DRAW:
 		case PD_MENU:
 		case PD_MENUITEM:
+		case PD_BITMAP:
 			break;
 		}
 }
@@ -736,10 +695,6 @@ EXPORT long ParamUpdate(
 			break;
 		case PD_COLORLIST:
 			dc = wColorSelectButtonGetColor( (wButton_p)p->control );
-#ifdef LATER
-			inx = wListGetIndex( (wList_p)p->control );
-			dc = colorTab[inx].color;
-#endif
 			if (dc != p->oldD.dc) {
 				p->oldD.dc = dc;
 				if ( /*(p->option&PDO_NOUPDUPD)==0 &&*/ p->valueP)
@@ -775,8 +730,20 @@ EXPORT long ParamUpdate(
 				if (p->oldD.s)
 					MyFree( p->oldD.s );
 				p->oldD.s = MyStrdup( stringV );
-				if ( /*(p->option&PDO_NOUPDUPD)==0 &&*/ p->valueP)
-					strcpy( (char*)p->valueP, stringV );
+				if ( /*(p->option&PDO_NOUPDUPD)==0 &&*/ p->valueP) {
+					if (p->context) {
+						strncpy((char*)p->valueP, stringV, (intptr_t)p->context);
+						((char *)p->valueP)[(intptr_t)p->context - 1] = '\0';
+						if (strlen(stringV) > (uintptr_t)p->context) {
+							NoticeMessage2(0, MSG_ENTERED_STRING_TRUNCATED, _("Ok"), NULL, (intptr_t)p->context);
+						}
+
+					}
+					else {
+						strcpy((char*)p->valueP, stringV);
+					}
+				}
+
 				if ( (p->option&PDO_NOUPDACT)==0 && pg->changeProc)
 					 pg->changeProc( pg, inx, CAST_AWAY_CONST stringV );
 				change |= (1L<<inx);
@@ -788,6 +755,7 @@ EXPORT long ParamUpdate(
 		case PD_TEXT:
 		case PD_MENU:
 		case PD_MENUITEM:
+		case PD_BITMAP:
 			break;
 		}
 	}
@@ -830,10 +798,6 @@ EXPORT void ParamLoadData(
 			break;
 		case PD_COLORLIST:
 			*(wDrawColor*)p->valueP = wColorSelectButtonGetColor( (wButton_p)p->control );
-#ifdef LATER
-			inx = wListGetIndex( (wList_p)p->control );
-			*(wDrawColor*)p->valueP = colorTab[inx].color;
-#endif
 			break;
 		case PD_FLOAT:
 			if (p->option & PDO_DIM) {
@@ -856,6 +820,7 @@ EXPORT void ParamLoadData(
 		case PD_TEXT:
 		case PD_MENU:
 		case PD_MENUITEM:
+		case PD_BITMAP:
 			break;
 		}
 	}
@@ -964,6 +929,7 @@ static long ParamIntRestore(
 		case PD_TEXT:
 		case PD_MENU:
 		case PD_MENUITEM:
+		case PD_BITMAP:
 			break;
 		}
 	}
@@ -1013,6 +979,7 @@ static void ParamIntSave(
 			case PD_TEXT:
 			case PD_MENU:
 			case PD_MENUITEM:
+			case PD_BITMAP:
 				break;
 			}
 		}
@@ -1108,7 +1075,7 @@ EXPORT void ParamRegister( paramGroup_p pg )
 		case PD_COMBOLIST:
 			if ( (p->option&PDO_LISTINDEX) ) {
 				if (!wPrefGetInteger( PREFSECT, prefName1, &valL, *(wIndex_t*)p->valueP ))
-					wPrefGetInteger( prefSect2, prefName2, &valL, *(wIndex_t*)p->valueP );
+					wPrefGetInteger( prefSect2, prefName2, &valL, valL );
 				if ( p->control )
 					wListSetIndex( (wList_p)p->control, (wIndex_t)valL );
 				*(wIndex_t*)p->valueP = (wIndex_t)valL;
@@ -1287,6 +1254,7 @@ EXPORT void ParamGroupRecord(
 		case PD_TEXT:
 		case PD_MENU:
 		case PD_MENUITEM:
+		case PD_BITMAP:
 			break;
 		}
 	}
@@ -1380,7 +1348,7 @@ static void ParamIntegerPush( const char * val, void * dp )
 	while ( isspace((unsigned char)*val)) val++;
 	valL = strtol( val, &cp, 10 );
 
-	wControlSetBalloon( p->control, 0, -5, NULL );
+	//wControlSetBalloon( p->control, 0, -5, NULL );
 	if ( val == cp ) {
 		wControlSetBalloon( p->control, 0, -5, _("Invalid Number") );
 		return;
@@ -1432,7 +1400,7 @@ static void ParamFloatPush( const char * val, void * dp )
 		if (p->option & PDO_ANGLE)
 			valF = NormalizeAngle( (angleSystem==ANGLE_POLAR)?valF:-valF );
 	}
-	wControlSetBalloon( p->control, 0, -5, NULL );
+	// wControlSetBalloon( p->control, 0, -5, NULL );
 	if ( !valid ) {
 		wControlSetBalloon( p->control, 0, -5, decodeErrorStr );
 		return;
@@ -1500,22 +1468,7 @@ static void ParamListPush( wIndex_t inx, const char * val, wIndex_t op, void * d
 			p->group->changeProc( p->group, p-p->group->paramPtr, &valL );
 		}
 		break;
-#ifdef LATER
-	case PD_COLORLIST:
-		dc = colorTab[inx].color;
-		rgb = wDrawGetRGB( dc );
-		if (recordF && (p->option&PDO_NORECORD)==0 && p->group->nameStr && p->nameStr) {
-			fprintf( recordF, "PARAMETER %s %s %ld\n",
-					p->group->nameStr, p->nameStr, rgb );
-			fflush( recordF );
-		}
-		if ( (p->option&PDO_NOPSHUPD)==0 && p->valueP)
-			*(wDrawColor*)(p->valueP) = dc;
-		if ( (p->option&PDO_NOPSHACT)==0 && p->group->changeProc ) {
-			; /* COLOR NOP */
-		}
-		break;
-#endif
+
 	default:
 		;
 	}
@@ -1634,7 +1587,7 @@ EXPORT void ParamChange( paramData_p p )
 	case PD_DROPLIST:
 	case PD_COMBOLIST:
 		if (recordF && (p->option&PDO_NORECORD)==0 && p->group->nameStr && p->nameStr) {
-			fprintf( recordF, "PARAMETER %s %s %d %s\n", p->group->nameStr, p->nameStr, *(wIndex_t*)p->valueP, ??? );
+			fprintf( recordF, "PARAMETER %s %s %d %s\n", p->group->nameStr, p->nameStr, *(wIndex_t*)p->valueP, "???" );
 		}
 #ifdef LATER
 		if ( p->control && (p->option&PDO_NOCONTUPD) == 0 )
@@ -1644,10 +1597,6 @@ EXPORT void ParamChange( paramData_p p )
 	case PD_COLORLIST:
 		if (recordF && (p->option&PDO_NORECORD)==0 && p->group->nameStr && p->nameStr)
 			fprintf( recordF, "PARAMETER %s %s %ld\n", p->group->nameStr, p->nameStr, rgb );
-#ifdef LATER
-		if ( p->control && (p->option&PDO_NOCONTUPD) == 0 )
-			wColorSelectButtonSetColor( (wButton_p)p->control, wDrawFindRGB(rgb) );
-#endif
 		break;
 	case PD_FLOAT:
 		tmpR = *(FLOAT_T*)p->valueP;
@@ -1842,13 +1791,6 @@ static void ParamPlayback( char * line )
 				dc = wDrawFindColor( rgb );
 				if ( p->control)
 					wColorSelectButtonSetColor( (wButton_p)p->control, dc );
-#ifdef LATER
-				valL = ColorTabLookup( dc );
-				if (p->control) {
-					wListSetIndex( (wList_p)p->control, (wIndex_t)valL );
-					wFlush();
-				}
-#endif
 				if (p->valueP)
 					*(wDrawColor*)p->valueP = dc;
 				if (pg->changeProc) {
@@ -1904,6 +1846,7 @@ static void ParamPlayback( char * line )
 			case PD_MESSAGE:
 			case PD_TEXT:
 			case PD_MENU:
+			case PD_BITMAP:
 				break;
 			case PD_MENUITEM:
 				if (p->valueP) {
@@ -2054,6 +1997,7 @@ static void ParamCheck( char * line )
 			case PD_TEXT:
 			case PD_MENU:
 			case PD_MENUITEM:
+			case PD_BITMAP:
 				break;
 		}
 		if ( hasError ) {
@@ -2093,7 +2037,6 @@ static void ParamCreateControl(
 	paramTextData_t * textDataP;
 	paramListData_t * listDataP;
 	wIcon_p iconP;
-    wDrawColor color = wDrawColorBlack;
 
 	wWin_p win;
 	wPos_t w;
@@ -2177,8 +2120,8 @@ static void ParamCreateControl(
 			listDataP->height = wControlGetHeight( pd->control );
 			break;
 		case PD_COLORLIST:
-			pd->control = (wControl_p)wColorSelectButtonCreate( win, xx, yy, helpStr, _(pd->winLabel), pd->winOption, 0, &color, ParamColorSelectPush, pd );
-			break;
+			pd->control = (wControl_p)wColorSelectButtonCreate( win, xx, yy, helpStr, _(pd->winLabel), pd->winOption, 0, NULL, ParamColorSelectPush, pd );
+            break;
 		case PD_MESSAGE:
 			if ( pd->winData != 0 )
 				w = (wPos_t)(long)pd->winData;
@@ -2440,10 +2383,6 @@ static void LayoutControls(
 			char * cp;
 			strcpy( message, pd->nameStr );
 			for ( cp=message; *cp; cp++ ) if ( *cp == '-' ) *cp = '_';
-			LOG( log_hotspot, 1, ( "popup %d %d %d %d _%s_%s\n",
-				controlK.orig.x+hotspotOffsetX, controlK.orig.y+hotspotOffsetY,
-				controlSize_x, controlSize_y,
-				group->nameStr, message ) )
 		}
 		/*
 		 * Set column term
@@ -2565,9 +2504,21 @@ static void ParamDlgProc(
 	}
 }
 
+/**
+ * Create a dialog box from data definition. 
+ *
+ * \param IN group	data definition for the dialog
+ * \param IN title  title of the new dialog
+ * \param IN okLabel text for the affirmative button
+ * \param IN okProc	subroutine to call when ok is pressed
+ * \param IN cancelProc if not NULL a subroutine for Cancel event. If NULL no cancel button is created
+ * \param IN needHelpButton if TRUE a help button is created
+ * \param IN layoutProc ???
+ * \param IN winOption ???
+ * \param IN changeProc ???
+ */
 
-
-EXPORT wWin_p ParamCreateDialog(
+wWin_p ParamCreateDialog(
 		paramGroup_p group,
 		char * title,
 		char * okLabel,
@@ -2580,7 +2531,6 @@ EXPORT wWin_p ParamCreateDialog(
 {
 	char helpStr[STR_SHORT_SIZE];
 	wPos_t w0, h0;
-	wButton_p lastB = NULL;
 	char * cancelLabel = (winOption&PD_F_ALT_CANCELLABEL?_("Close"):_("Cancel"));
 
 	winOption &= ~PD_F_ALT_CANCELLABEL;
@@ -2593,34 +2543,22 @@ EXPORT wWin_p ParamCreateDialog(
 	if ( (winOption&F_RESIZE) != 0 )
 		winOption |= F_RECALLSIZE;
 
-	sprintf( helpStr, "cmd%s", group->nameStr );
-	helpStr[3] = toupper((unsigned char)helpStr[3]);
-
 	group->win = wWinPopupCreate( mainW, DlgSepRight, DlgSepFrmBottom, helpStr, title, group->nameStr, F_AUTOSIZE|winOption, ParamDlgProc, group );
 
 	if ( okLabel && okProc ) {
 		sprintf( helpStr, "%s-ok", group->nameStr );
-		lastB = group->okB = wButtonCreate( group->win, 0, 0, helpStr, okLabel, BB_DEFAULT, 0, (wButtonCallBack_p)ParamButtonOk, group );
+		group->okB = wButtonCreate( group->win, 0, 0, helpStr, okLabel, BB_DEFAULT, 0, (wButtonCallBack_p)ParamButtonOk, group );
 	}
 	if ( group->cancelProc ) {
-		lastB = group->cancelB = wButtonCreate( group->win, 0, 0, NULL, cancelLabel, BB_CANCEL, 0, (wButtonCallBack_p)ParamButtonCancel, group );
+		group->cancelB = wButtonCreate( group->win, 0, 0, NULL, cancelLabel, BB_CANCEL, 0, (wButtonCallBack_p)ParamButtonCancel, group );
 	}
 	if ( needHelpButton ) {
 		sprintf( helpStr, "cmd%s", group->nameStr );
 		helpStr[3] = toupper((unsigned char)helpStr[3]);
-		lastB = group->helpB = wButtonCreate( group->win, 0, 0, NULL, _("Help"), BB_HELP, 0, (wButtonCallBack_p)wHelp, MyStrdup(helpStr) );
+		group->helpB = wButtonCreate( group->win, 0, 0, NULL, _("Help"), BB_HELP, 0, (wButtonCallBack_p)wHelp, MyStrdup(helpStr) );
 	}
 
-	LOG( log_hotspot, 1, ( "mkshg ${PNG2DIR}/%s.png ${SHGDIR}/%s.shg << EOF\n", group->nameStr, group->nameStr ) )
 	LayoutControls( group, ParamCreateControl, &group->origW, &group->origH );
-	if ( group->okB )
-		LOG( log_hotspot, 1, ( "popup %d %d %d %d _%s_%s\n",
-				wControlGetPosX((wControl_p)(group->okB))+hotspotOffsetX,
-				wControlGetPosY((wControl_p)(group->okB))+hotspotOffsetY,
-				wControlGetWidth((wControl_p)(group->okB)),
-				wControlGetHeight((wControl_p)(group->okB)),
-				group->nameStr, "ok" ) )
-	LOG( log_hotspot, 1, ( "EOF\n" ) )
 
 	group->origW += DlgSepRight;
 	group->origH += DlgSepBottom;
@@ -2630,11 +2568,9 @@ EXPORT wWin_p ParamCreateDialog(
 			 group->origH != h0 ) {
 			LayoutControls( group, ParamPositionControl, NULL, NULL );
 		}
-	} else if ( group->origW > w0 || group->origH > h0 ) {
-		if ( group->origW > w0 )
-			w0 = group->origW;
-		if ( group->origH > h0 )
-			h0 = group->origH;
+	} else {
+		w0 = max(group->origW, w0);
+		h0 = max(group->origH, h0);
 		wWinSetSize( group->win, w0, h0 );
 	}
 
@@ -2694,6 +2630,5 @@ EXPORT void ParamInit( void )
 {
 	AddPlaybackProc( "PARAMETER", ParamPlayback, NULL );
 	AddPlaybackProc( "PARAMCHECK", ParamCheck, NULL );
-	log_hotspot = LogFindIndex( "hotspot" );
 	log_paramLayout = LogFindIndex( "paramlayout" );
 }
